#define OEMRESOURCE 
#include <Windows.h>

#include "resource.h"

#include <math.h>

#define PI 3.1415926535897932384626433832795028841971693993751058209749445923078164062

int WINAPI wWinMain( HINSTANCE hInstance, HINSTANCE, PWSTR, int nCmdShow )
{
    HCURSOR defaultCursor = LoadCursor( NULL, IDC_ARROW );
    HCURSOR defaultHand = LoadCursor( NULL, IDC_HAND );
    HCURSOR defaultCursorCopy = CopyCursor( defaultCursor );
    HCURSOR defaultHandCopy = CopyCursor( defaultHand );

    int const numCursors = 360;
    HCURSOR cursorResource[ numCursors ];
    HCURSOR handResource[ numCursors ];
    for ( int i = 0; i < numCursors; ++i )
    {
        cursorResource[ i ] = LoadCursor( hInstance, MAKEINTRESOURCE( IDC_CURSOR1 + i ) );
        handResource[ i ] = LoadCursor( hInstance, MAKEINTRESOURCE( IDC_CURSOR1 + i + numCursors ) );
    }

    int prevIndex = -1;
    tagPOINT prevPoint = { 0, 0 };
    double xDir = 1.0f, yDir = 0.0f;
    while ( true )
    {
        tagPOINT point;
        if ( GetPhysicalCursorPos( &point ) )
        {
            double const scalar = 0.025f;
            xDir += double( point.x - prevPoint.x ) * scalar;
            yDir += double( point.y - prevPoint.y ) * scalar;
            prevPoint = point;

            double len = sqrt( xDir * xDir + yDir * yDir );
            if ( len > 0.00001 )
            {
                xDir /= len;
                yDir /= len;
            }
            else
            {
                xDir = 1.0;
                yDir = 0.0;
            }

            double const rot = atan2( -yDir, xDir );
            int index = int( ( rot + 2.0 * PI + PI / double( numCursors ) ) / ( 2.0 * PI / double( numCursors ) ) ) % numCursors;
            if ( index != prevIndex )
            {
                HCURSOR cursor = CopyCursor( cursorResource[ index ] );
                SetSystemCursor( cursor, OCR_NORMAL );
                HCURSOR hand = CopyCursor( handResource[ index ] );
                SetSystemCursor( hand, OCR_HAND );
                prevIndex = index;
            }
        }

        if ( GetAsyncKeyState( VK_CONTROL ) && GetAsyncKeyState( VK_MENU ) && GetAsyncKeyState( VK_DELETE ) )
        {
            SetSystemCursor( defaultCursorCopy, OCR_NORMAL );
            SetSystemCursor( defaultHandCopy, OCR_HAND );
            defaultCursorCopy = CopyCursor( defaultCursor );
            defaultHandCopy = CopyCursor( defaultHand );
        }

        if ( GetAsyncKeyState( VK_CONTROL ) && GetAsyncKeyState( VK_SHIFT ) && GetAsyncKeyState( VK_MENU ) )
        {
            break;
        }

        Sleep( 16 );
    }

    SetSystemCursor( defaultCursorCopy, OCR_NORMAL );
    SetSystemCursor( defaultHandCopy, OCR_HAND );
    Sleep( 1000 ); // Wait a bit for the cursor to be reverted

    return 0;
}